from botworks.chat.Response import Response
from botworks.chat.InteractionDefinition import InteractionDefinition, Or

messages = [
    InteractionDefinition(
        trigger_definition=Or("spoiler"),
        mod_exempt=False,
        cooldown_duration_seconds=3600,
        response=Response(
            text=["Rosebud was his sled", "Snape kills Dumbledore",
                  "Bruce Willis was dead the whole time", "Water kills the aliens",
                  "Vader is Luke's father", "Leeloo is the fifth element",
                  "Jack dies", "Mission Possible", "Harry kills Voldemort",
                  "Tyler Durden is not real", "299 Die", "Neo is the one. Also maybe Jesus.",
                  "Orion is the cat, the galaxy is a marble", "Leonard killed his own wife",
                  "http://i.imgur.com/BrV6KJT.jpg", "The village is in a modern nature preserve",
                  "Soylent green is made of people", "Judge Doom framed Roger Rabbit",
                  "Borden had an identical twin disguised as Fallon",
                  "Andy used the poster to hide a hole he was digging",
                  "Elijah caused the train crash to see if superheros were real",
                  "Il Duce is Conner and Murphy's father, Agent Smecker joins their cause",
                  "It was all part of the Rekall package",
                  "Grace and her children are the ghosts", "Leia is Luke's sister",
                  "Sergean Howie was lured to the island as a sacrifice", "Deckard was a replicant... maybe",
                  "Everyone on the island is dead and trapped in some sort of time-travelling purgatory"])
    ),
    InteractionDefinition(
        trigger_definition=Or("night king", "nightking"),
        mod_exempt=False,
        cooldown_duration_seconds=0,
        response=Response(emoji="nightking")
    )
]


import logging
import time

from slackclient import SlackClient

log = logging.getLogger(__name__)
log.setLevel(logging.INFO)

BOT_NAME = "autodesktest"


class Clack:
    def __init__(self, bot_token, bot_name, config=None):
        if config is None:
            config = {}
        obfuscated_token = bot_token[:5] + "*********************************" + bot_token[-4:]
        log.error("Got token " + obfuscated_token)
        self.__slack_client = SlackClient(bot_token)
        self.__bot_token = bot_token
        self.__admin_channel = None
        if not self.__slack_client.rtm_connect():
            log.error("Failed to initialize")
            exit(1)

        self.__user_list_refresh_time = 0
        self.__channel_list_refresh_time = 0
        self.__deleted_user_list = []
        self.__complete_user_list = None
        self.__complete_channel_list = None
        self.__active_human_user_list = []

        self.__complete_user_list = self.all_users()
        self.__complete_channel_list = self.all_channels()
        log.info("User list size: " + str(len(self.__complete_user_list)))
        log.info("DeletedUserList : " + str(len(self.__deleted_user_list)))
        log.info("channel list size: " + str(len(self.__complete_channel_list)))
        bot = self.find_user_by_name(bot_name)
        self.__myId = bot['id']
        if config and 'log_level' in config:
            log.setLevel(config['log_level'])

    def get_bot_info(self):
        return {"id": self.__myId, "token": self.__bot_token}

    def read(self):
        return self.__slack_client.rtm_read()

    def get_channel_member_count(self, channel_id):
        try:
            channel_members = self.__slack_client.api_call("channels.info",
                                                           channel=channel_id).get('channel').get('members')
            cleaned_list = list(set(channel_members) - set(self.__active_human_user_list))
            return len(cleaned_list)
        except Exception as e:
            log.error(e)
            log.error(e.__traceback__)
            log.warning(self.__slack_client.api_call("channels.info", channel=channel_id))
            return "1"

    def post_message(self, channel, message, send_delete_message=True):
        x = self.__slack_client.api_call("chat.postMessage", channel=channel,
                                         text=message, as_user=True)
        if send_delete_message and self.__admin_channel:
            self.post_message(self.__admin_channel, message + "\n!delete " + str(x['channel']) + " " + str(x['ts']),
                              send_delete_message=False)

    def react(self, payload, reaction: str):
        log.info("reaction: " + str(
            self.__slack_client.api_call("reactions.add",
                                         name=reaction, channel=payload.channel,
                                         timestamp=payload.timestamp)
        ))

    def ephemeral_message(self, payload, message: str):
        log.info(
            self.__slack_client.api_call("chat.postEphemeral",
                                         channel=payload.channel, user=payload.user,
                                         text=message, as_user=True)
        )

    def thread_reply(self, payload, message: str, send_delete_message=True):
        ts = payload.timestamp
        if 'thread_ts' in payload.raw:
            ts = payload.raw['thread_ts']
        x = self.__slack_client.api_call("chat.postMessage", channel=payload.channel,
                                         text=message, thread_ts=ts, as_user=True)
        if send_delete_message and self.__admin_channel:
            self.post_message(self.__admin_channel, message + "\n!delete " + str(x['channel']) + " " + str(x['ts']),
                              send_delete_message=False)

    def all_users(self):
        if self.__user_list_refresh_time > time.time():
            return self.__complete_user_list
        api_call = self.__slack_client.api_call("users.list")
        if api_call.get('ok'):
            self.__user_list_refresh_time = int(time.time() + 6000)
            deleted_users = filter(lambda m: 'deleted' in m and m['deleted'] and 'id' in m, api_call.get('members'))
            self.__deleted_user_list = list(deleted_users)
            self.__active_human_user_list = [user['id'] for user in api_call.get('members')
                                             if not user['deleted'] and not user['is_bot']]
            return api_call.get('members')
        return self.__complete_user_list

    def all_channels(self):
        if self.__channel_list_refresh_time > time.time():
            return self.__complete_channel_list
        api_call = self.__slack_client.api_call("channels.list")
        if api_call.get('ok'):
            self.__channel_list_refresh_time = int(time.time() + 6000)
            return api_call.get('channels')
        return self.__complete_channel_list

    def find_user_by_name(self, search_term: str):
        all_users = self.all_users()
        name = search_term.lower()
        if not all_users:
            return None
        for u in all_users:
            if ('real_name' in u and u['real_name'].lower() == name) \
                    or ('name' in u and u['name'].lower() == name) \
                    or ('id' in u and u['id'].lower == name)\
                    or ('profile' in u and (
                            ('display_name_normalize' in u['profile'] and u['profile']['display_name_normalize'] == name) or
                            ('display_name' in u['profile'] and u['profile']['display_name'] == name))):
                return u
        return None

    def get_user_by_id(self, user_id: str):
        return self.__slack_client.api_call("users.info", user=user_id)

    def find_channel_info(self, channel_name: str):
        lower_channel = channel_name.lower()
        channels = self.all_channels()
        for c in channels:
            if ('name' in c and c['name'].lower() == lower_channel) or ('id' in c and c['id'].lower() == lower_channel):
                return c
        return None

    def list_im(self):
        return self.__slack_client.api_call("im.list")

    def delete_message(self, channel, ts):
        channel = channel.upper()
        log.debug("deleting " + channel + " " + ts)
        log.debug(self.__slack_client.api_call("chat.delete", ts=ts, channel=channel))

    # def leave(channel):  # bots can't do this.
    #     print(slack_client.api_call("channels.leave", channel=channel))



import requests
import logging
import pytesseract

from io import BytesIO

from PIL import Image

from botworks.chat.EventTypes import user_typing_ignored

log = logging.getLogger(__name__)
log.setLevel(logging.INFO)


class Payload:
    def __init__(self):
        self.user = None
        self.raw = None
        self.lower_message = None
        self.event_type = None
        self.channel = None
        self.timestamp = None
        self.is_direct_message = None
        self.isTargetedMessage = None
        self.imageText = None
        self.sharedMessage = None

    def finalize(self, rtm_message, bot_id: str, bot_token: str):
        self.raw = rtm_message
        self.user = rtm_message['user']
        self.lower_message = rtm_message['text'].lower() if 'text' in rtm_message else ""
        self.event_type = rtm_message['type'].lower() if 'type' in rtm_message else ""

        channel_field = "channel"
        if 'channel_id' in rtm_message:
            channel_field = 'channel_id'
        log.debug("Using " + channel_field + " as " + rtm_message[channel_field])
        self.channel = rtm_message[channel_field]
        self.timestamp = rtm_message['ts']
        self.is_direct_message = str(rtm_message['channel']).startswith("D")
        self.isTargetedMessage = self.is_direct_message
        if str(self.lower_message).startswith("<@" + str(bot_id) + ">"):
            self.isTargetedMessage = True
            self.lower_message = str(self.lower_message).replace("<@" + str(bot_id) + ">", "", 1).strip()

        if 'file' in rtm_message and bot_token:
            try:
                url = rtm_message['file']['url_private_download']
                log.info("Fetching image url: " + url)
                auth_header = {"Authorization": "Bearer " + bot_token}
                string = BytesIO(requests.get(url, headers=auth_header).content)
                img = Image.open(string)
                decoded_text = pytesseract.image_to_string(img)
                self.imageText = decoded_text

            except Exception as e:
                log.error("Error parsing image text: " + str(e))
                self.imageText = None
        else:
            self.imageText = None

        if 'attachments' in rtm_message and len(rtm_message['attachments']) > 0:
            self.sharedMessage = rtm_message['attachments'][0]
        else:
            self.sharedMessage = None


class PayloadFactory:
    def __init__(self, bot_id: str, bot_token: str, config=None):
        if config is None:
            config = {}
        self.__config = config
        self.__botId = bot_id
        self.__botToken = bot_token
        if 'log_level' in config:
            log.setLevel(config['log_level'])

    def parse_payload(self, message):
        if self.valid_message(message):
            log.debug(message)
            return Payload().finalize(message, self.__botId, self.__botToken)
        return None

    @staticmethod
    def valid_message(message):
        return message and ('bot_id' not in message or not message['bot_id'])\
               and ('is_ephemeral' not in message or not message['is_ephemeral'])  \
               and (('text' in message or 'type' in message and not message['type'] == user_typing_ignored)
                    and 'user' in message and 'channel' in message)




import logging

import time
from typing import List

from botworks.channel.Channel import Channel
from botworks.config_constants import SLEEP_TIME, ERROR_SLEEP_TIME, LOG_LEVEL, HEALTHY_HANDLER, UNHEALTHY_HANDLER
from botworks.slack.Clack import Clack
from botworks.slack.MessageHandler import MessageHandler

log = logging.getLogger(__name__)
log.setLevel(logging.WARNING)


class Botworks:
    def __init__(self, bot_token, bot_name, channels: List[Channel], config=None):
        if config is None:
            config = {}
        if not config[SLEEP_TIME]:
            config[SLEEP_TIME] = 5
        if not config[ERROR_SLEEP_TIME]:
            config[ERROR_SLEEP_TIME] = 10
        if not config[LOG_LEVEL]:
            config[LOG_LEVEL] = logging.WARNING
        self.botToken = bot_token
        self.botName = bot_name
        self.channels = channels
        self.conf = config
        self.healthy = True
        log.setLevel(config[LOG_LEVEL])
        for c in self.channels:
            for r in c.responses:
                r.finalize(self.conf)

    def listen(self):
        clacker = Clack(self.botToken, self.botName, config=self.conf)
        for c in self.channels:
            c.channelId = next(slack_channel for slack_channel in clacker.all_channels()
                               if slack_channel['name'] and slack_channel['name'] == c.channelName)['id']
            log.info("Monitoring " + c.channelName + " as " + c.channelId)
            for mod_name in c.mod_names:
                mod_user = clacker.find_user_by_name(mod_name)
                if mod_user and 'id' in mod_user:
                    c.mod_ids.append(mod_user['id'])

        message_handler = MessageHandler(clacker, channels=self.channels, config=self.conf)
        while True:
            try:
                message_handler.parse_slack_output(clacker.read())
                if not self.healthy:
                    self.healthy = True
                    if self.conf[HEALTHY_HANDLER]:
                        self.conf[HEALTHY_HANDLER]()
                time.sleep(self.conf[SLEEP_TIME])
            except Exception as e:
                log.error("Major error")
                log.error(e)
                self.healthy = False

                if self.conf[UNHEALTHY_HANDLER]:
                    self.conf[UNHEALTHY_HANDLER](error=e)
                time.sleep(self.conf[ERROR_SLEEP_TIME])
